
local tw			 	= 32
local player, map = {}, {}


---------------------------------------------------------------------------------------------
-- Map --
---------------------------------------------------------------------------------------------

function map:init()
	self.level 		= {
		{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,},
		{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,},
		{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,},
		{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,},
		{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,},
		{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,},
		{1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1,},
		{1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1,},
		{1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1,},
		{1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1,},
		{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,},
		{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,},
		{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,},
		{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,},
		{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,},
	}

	function map:renderCanvas()
		love.graphics.setCanvas(self.canvas)
		love.graphics.clear()
		love.graphics.setBlendMode("alpha", "premultiplied")
		for y=1, #self.level do
			for x=1, #self.level[1] do
				if self.level[y][x] == 1 then -- solid
					love.graphics.rectangle("fill", x*tw-tw, y*tw-tw, tw, tw)
				end
			end
		end
		love.graphics.setBlendMode("alpha")
		love.graphics.setCanvas()
	end

	function map:getTile(x, y)
		if self.level[y] and self.level[y][x] then
			return self.level[y][x]
		end
	end

	function map:draw()
		love.graphics.draw(self.canvas)
	end

	-- Init --
	self.canvas = love.graphics.newCanvas()
	self:renderCanvas()
end



---------------------------------------------------------------------------------------------
-- Player --
---------------------------------------------------------------------------------------------

function player:init(x, y)

	function player:move(dx, dy)
		local newx, newy = self.x+dx, self.y+dy
		local tile = map:getTile(newx, newy)
		if tile and tile == 0 then
			self.x = newx
			self.y = newy
		end
	end

	function player:draw()
		love.graphics.setColor(255, 50, 50, 255)
		love.graphics.rectangle("fill", self.x*tw-tw, self.y*tw-tw, tw, tw)
		love.graphics.setColor(255, 255, 255, 255)
	end

	self.x = x or 2
	self.y = y or 2
end



---------------------------------------------------------------------------------------------
-- Main Callbacks --
---------------------------------------------------------------------------------------------

function love.load()
	-- Map --
	map:init()

	-- Player --
	player:init()
end



function love.update(dt)
end



function love.draw()
	map:draw()
	player:draw()
end



function love.keypressed(key)
	if key == "left" then player:move(-1, 0)
	elseif key == "right" then player:move(1, 0)
	elseif key == "up" then player:move(0, -1)
	elseif key == "down" then player:move(0, 1)
	elseif key == "escape" then love.event.quit() end
end
